#!/usr/bin/env python

import os

def test_grep_shell():

    cmd = "echo baseball basketball rugby soccer | fmt -1 | grep2 -n 1 'european sport,american sport'"
    out = os.popen(cmd).read().splitlines()

    assert out == [
        'american sport:baseball',
        'american sport:basketball',
        'european sport:rugby',
        'european sport:soccer',
    ]



__all__ = [
    'Attn',
    'Grep',
    'grep',
    'greps',
]

import assure
import embd

SIMILARITY_DEFINITIONS = [
    'a @ b',
    'a @ b(a)',
    'a @ b(b)',
    'a(a) @ b',
    'a(a) @ b(a)',
    'a(a) @ b(b)',
    'a(b) @ b',
    'a(b) @ b(a)',
    'a(b) @ b(b)',
]


class Attn:

    def __init__(self, queries, keys, *, sims='a @ b'):

        q = embd.promote(queries)
        k = embd.promote(keys)

        if sims not in self.sims_defs():
            raise ValueError(f"sims must be one of: {self.sims_defs()}")

        s = self.sims(q, k, sims)

        import numpy as np
        i = np.argsort(-s)

        self.queries = queries
        self.keys = keys
        self.q = q
        self.k = k
        self.s = s
        self.i = i

    @classmethod
    def all(cls, queries, keys, *, sims='a @ b', **kwds):
        """ Evaluate possible definitions of similarity """
        return [cls(queries, keys, sims=sims, **kwds) for sims in cls.sims_defs()]

    @classmethod
    def sims_defs(cls):
        return SIMILARITY_DEFINITIONS

    @classmethod
    def sims(cls, q, k, sims):
        return (lambda a,b: eval(sims))(q,k)

    def __repr__(self):
        return repr(self.s)


class Grep(Attn):

    def __init__(self, queries, keys, *, sims='a @ b', n=None):

        super().__init__(queries, keys, sims=sims)
        s = self.s
        i = self.i

        g = 0*s
        g.iloc[:, :] = s.columns.values[i]
        g.columns = list(range(1, len(g.columns)+1))
        g.columns.name = sims
        g = g.iloc[:, 0:n]

        # attributes
        self.i = i
        self.g = g

    def __repr__(self):
        return repr(self.g)

    def __getitem__(self, n):
        return self.g[n]

    def what(self, n=None):
        l = self.g.values.tolist()
        import numpy as np
        return np.array([a[:n] for a in l])

    def fmt_what(self, n=None):
        ll = self.what(n=n)
        l = [','.join(a) for a in ll]
        return '\n'.join(l)

    def fmt_grep(self, n=1):
        return self.csv(n=n)

    def csv(self, n=1):
        import pandas as pd
        g = self.g
        g = g.iloc[:, 0:n]
        qs = g.index.tolist()
        Ks = g.values
        lines = []
        for q,K in zip(qs,Ks):
            k = ','.join(K)
            line = f"{k}:{q}"
            lines.append(line)
        o = '\n'.join(lines)
        return o

    def tolist(self):
        return self.g.values.ravel().tolist()

    def intersect(self, s):
        return set(self.tolist()) & set(s)

grep = Grep
greps = Grep.all



#!/usr/bin/python3

from wnix import grep, greps

# queries: aka, questions.
# There are arbitrarily many of these,
# and we never know all of them,
# so any meaningful operation should be a map over queries,
# we should never (for example) move to the center of
# mass of the set of queries.

QUERIES_1 = [
    'a guy pretending to be a cat',
    'a drunk guy who thinks hes a lion',
    'your submissive furry boyfriend',
    'a male biological cat',
    'your obedient pet tabby mister whiskers',
    'a young male tiger at the San Diego zoo',
]

QUERIES_2 = [
    'a quirky tinder match named jane who meows at you',
    'an obnoxious e-girl who wears ears',
    'your submissive furry girlfriend',
    'your fierce pet tabby mrs boots',
    'a young female tiger at the Omaha zoo',
    'an african jaguar mother who recently had cubs',
]

QUERIES_3 = QUERIES_1 + QUERIES_2

KEYS_1 = ['cat boy', 'boy cat']
KEYS_2 = ['cat girl', 'girl cat']
KEYS_3 = KEYS_1 + KEYS_2

ANSWERS_1 = ['cat boy', 'cat boy', 'cat boy', 'boy cat', 'boy cat', 'boy cat']
ANSWERS_2 = ['cat girl', 'cat girl', 'cat girl', 'girl cat', 'girl cat', 'girl cat']
ANSWERS_3 = ANSWERS_1 + ANSWERS_2


def test_grep_cats_1(debug=False):
    gs = greps(QUERIES_1, KEYS_1, n=1)
    for g in gs:
        assert g[1].tolist() == ANSWERS_1
        if debug:
            print(g)
            print('='*42)

def test_grep_cats_2(debug=False):
    gs = greps(QUERIES_2, KEYS_2, n=1)
    for g in gs:
        assert g[1].tolist() == ANSWERS_2
        if debug:
            print(g)
            print('='*42)

def test_grep_cats_3(debug=False):
    gs = greps(QUERIES_3, KEYS_3, n=1)
    for g in gs:
        assert g[1].tolist() == ANSWERS_3
        if debug:
            print(g)
            print('='*42)
